@@ -2,7 +2,9 @@ import re
 import itertools
 
 import wifi.subprocess_compat as subprocess
-from wifi.pbkdf2 import pbkdf2_hex
+from pbkdf2 import PBKDF2
+from wifi.utils import ensure_file_exists
+from wifi.exceptions import ConnectionError
 
 
 def configuration(cell, passkey=None):
@@ -17,9 +19,9 @@ def configuration(cell, passkey=None):
             'wireless-channel': 'auto',
         }
     else:
-        if cell.encryption_type == 'wpa2':
+        if cell.encryption_type.startswith('wpa'):
             if len(passkey) != 64:
-                passkey = pbkdf2_hex(passkey, cell.ssid, 4096, 32)
+                passkey = PBKDF2(passkey, cell.ssid, 4096).hexread(32)
 
             return {
                 'wpa-ssid': cell.ssid,
@@ -35,6 +37,9 @@ def configuration(cell, passkey=None):
             raise NotImplementedError
 
 
+bound_ip_re = re.compile(r'^bound to (?P<ip_address>\S+)', flags=re.MULTILINE)
+
+
 class Scheme(object):
     """
     Saved configuration for connecting to a wireless network.  This
@@ -44,6 +49,17 @@ class Scheme(object):
 
     interfaces = '/etc/network/interfaces'
 
+    @classmethod
+    def for_file(cls, interfaces):
+        """
+        A class factory for providing a nice way to specify the interfaces file
+        that you want to use.  Use this instead of directly overwriting the
+        interfaces Class attribute if you care about thread safety.
+        """
+        return type(cls)(cls.__name__, (cls,), {
+            'interfaces': interfaces,
+        })
+
     def __init__(self, interface, name, options=None):
         self.interface = interface
         self.name = name
@@ -66,8 +82,9 @@ class Scheme(object):
         """
         Returns an generator of saved schemes.
         """
+        ensure_file_exists(cls.interfaces)
         with open(cls.interfaces, 'r') as f:
-            return extract_schemes(f.read())
+            return extract_schemes(f.read(), scheme_class=cls)
 
     @classmethod
     def where(cls, fn):
@@ -96,12 +113,30 @@ class Scheme(object):
         """
         Writes the configuration to the :attr:`interfaces` file.
         """
-        assert not Scheme.find(self.interface, self.name)
+        assert not self.find(self.interface, self.name), "This scheme already exists"
 
         with open(self.interfaces, 'a') as f:
             f.write('\n')
             f.write(str(self))
 
+    def delete(self):
+        """
+        Deletes the configuration from the :attr:`interfaces` file.
+        """
+        iface = "iface %s-%s inet dhcp" % (self.interface, self.name)
+        content = ''
+        with open(self.interfaces, 'r') as f:
+            skip = False
+            for line in f:
+                if not line.strip():
+                    skip = False
+                elif line.strip() == iface:
+                    skip = True
+                if not skip:
+                    content += line
+        with open(self.interfaces, 'w') as f:
+            f.write(content)
+
     @property
     def iface(self):
         return '{0}-{1}'.format(self.interface, self.name)
@@ -116,15 +151,34 @@ class Scheme(object):
         """
         Connects to the network as configured in this scheme.
         """
-        subprocess.check_call(['/sbin/ifdown', self.interface])
-        subprocess.check_call(['/sbin/ifup'] + self.as_args())
+
+        subprocess.check_output(['/sbin/ifdown', self.interface], stderr=subprocess.STDOUT)
+        ifup_output = subprocess.check_output(['/sbin/ifup'] + self.as_args(), stderr=subprocess.STDOUT)
+
+        return self.parse_ifup_output(ifup_output)
+
+    def parse_ifup_output(self, output):
+        matches = bound_ip_re.search(output)
+        if matches:
+            return Connection(scheme=self, ip_address=matches.group('ip_address'))
+        else:
+            raise ConnectionError("Failed to connect to %r" % self)
+
+
+class Connection(object):
+    """
+    The connection object returned when connecting to a Scheme.
+    """
+    def __init__(self, scheme, ip_address):
+        self.scheme = scheme
+        self.ip_address = ip_address
 
 
 # TODO: support other interfaces
 scheme_re = re.compile(r'iface\s+(?P<interface>wlan\d?)(?:-(?P<name>\w+))?')
 
 
-def extract_schemes(interfaces):
+def extract_schemes(interfaces, scheme_class=Scheme):
     lines = interfaces.splitlines()
     while lines:
         line = lines.pop(0)
@@ -144,6 +198,6 @@ def extract_schemes(interfaces):
                 key, value = re.sub(r'\s{2,}', ' ', lines.pop(0).strip()).split(' ', 1)
                 options[key] = value
 
-            scheme = Scheme(interface, scheme, options)
+            scheme = scheme_class(interface, scheme, options)
 
             yield scheme
