@@ -1,5 +1,6 @@
-from __future__ import print_function, unicode_literals
+from __future__ import print_function, unicode_literals, division
 
+import os
 import sys
 
 
@@ -37,3 +38,20 @@ def print_table(matrix, sep='  ', file=sys.stdout, *args, **kwargs):
 
     for row in matrix:
         print(format.format(*row).strip(), file=file, *args, **kwargs)
+
+
+def db2dbm(quality):
+    """
+    Converts the Radio (Received) Signal Strength Indicator (in db) to a dBm
+    value.  Please see http://stackoverflow.com/a/15798024/1013960
+    """
+    dbm = int((quality / 2) - 100)
+    return min(max(dbm, -100), -50)
+
+
+def ensure_file_exists(filename):
+    """
+    http://stackoverflow.com/a/12654798/1013960
+    """
+    if not os.path.exists(filename):
+        open(filename, 'a').close()
